<?php
/**
 * @file
 * API functions for the ls_sync module
 */

/**
 * Return the active database connection name.
 *
 * Retrieve the active database connection name from the global variable
 * $active_db, using the db_set_active() Drupal function;
 *
 * @param $settings
 *   An array. Corresponding to the drupal variable ls_settings.
 *
 * @return
 *   A string, the active database name
 */
function limesurvey_sync_active_db_name() {
  global $db_url;
  if (!is_array($db_url)) {
    // No multiple connection.
    return 'default';
  }
  else {
    global $active_db;
    static $db_conn_names = array();
    static $store_act_db = array();
    if (empty($db_conn_names)) {
      $i = 0;
      foreach ($db_url as $human_name => $connect_url) {
        $previous_conn = db_set_active($human_name);
        if (empty($db_conn_names)) {
          $orig_conn = $previous_conn;
        }
        // $active_db is updated on each call of db_set_active().
        $store_act_db[$i] = $active_db;
        $db_conn_names[$i] = $human_name;
        $i++;
      }
      // switch back to the original connection.
      db_set_active($orig_conn);
    }
    $j = array_search($active_db, $store_act_db, TRUE);
    return $db_conn_names[$j];
  }
}

/**
 * Activate the LimeSurvey database.
 *
 * Function called before each query or group of querys on the Limesurvey
 * database.
 *
 * @return
 *   A string, the name of the previously active database, or FALSE if none
 *   was found.
 */
function limesurvey_sync_db_open() {
  // If $db_url['ls'] is not defined in the sites/default/settings.php file,
  // (case where the LimeSurvey database and the Drupal Database are identical,
  // by default) : the Drupal database is called.
  global $db_url;
  $current_db = limesurvey_sync_active_db_name();
  // Don't call the db_set_active() function if it not necessary.
  if (isset($db_url['ls']) && $current_db != 'ls') {
    return db_set_active('ls');
  }
  return $current_db;
}

/**
 * Desactivate the LimeSurvey database.
 *
 * Function called after each query or group of querys on the Limesurvey
 * database, to allow the system to switch back to the Drupal database or keep
 * staying on the LimeSurvey database depending on the previously active
 * database.
 *
 * @param $db_to_active
 *   A string. the name of the database connection to active on closing the
 *   LimeSurvey database connection. Default is 'default', corresponding to the
 *   Drupal system database.
 *
 * @return
 *   A string, the name of the current active database, after the
 *   limesurvey_sync_db_close() is called.
 */
function limesurvey_sync_db_close($db_to_active = 'default') {
  // Don't call the db_set_active() function if it not necessary.
  $current_db = limesurvey_sync_active_db_name();
  if ($current_db != $db_to_active) {
    db_set_active($db_to_active);
  }
  return $db_to_active;
}

/**
 * Provide the LimeSurvey database table name.
 *
 * Depending on the selected table, concatenate the LimeSurvey database table
 * name to the survey ID. For example,
 * limesurvey_sync_table('surveys', 12356) returns 'surveys_123456'.
 *
 * @param $table
 *   A string. Corresponding to the LimeSurvey table generic name. Default is
 *   'survey'.
 * @param $sid
 *   (optional) An integer. Corresponding to the survey ID reference. Default
 *   is empty.
 * @param $ls_settings
 *   (optional) An array. Corresponding to the LimeSurvey database connecting
 *   informations. Default is empty, current settings will be used.
 *
 * @return
 *   A string, corresponding to the LimeSurvey database name.
 */
function limesurvey_sync_table($table = 'surveys', $sid = '', $ls_settings = array()) {
  if (empty($ls_settings)) {
    $ls_settings = variable_get('ls_settings', array('ls_db_prefix' => 'lime_'));
  }
  switch ($table) {
    // Handle tokens and answers tables : they need the $sid parameter.
    case 'tokens':
    case 'survey':
    case 'answer':
      // Watch out ! The answerS table exists to in the Limesurvey schema !!
      if ($table == 'answer') {
        // Answer is an more explicit name for the survey answers table name.
        $table = 'survey';
      }
      $table_name = $ls_settings['ls_db_prefix'] . $table . '_' . (string) $sid;
      break;
    default:
      $table_name = $ls_settings['ls_db_prefix'] . $table;
  }
  return check_plain($table_name);
}


/**
 * Provide The LimeSurvey global settings.
 *
 * Two LimeSurvey global settings are used into the LimeSurvey Sync module :
 * 'timeadjust' for the Timezone (and date) features and 'DBVersion' for
 * handling various bugs or features depending on the LimeSurey build version.
 *
 * @return
 *   An array, the Limesurvey global settings values keyed fy there name, or
 *   FALSE if the settings_global page does not exist.
 */
function limesurvey_sync_ls_global_settings() {
  static $ls_gs = array();
  if (empty($ls_gs)) {
    if (limesurvey_sync_db_table_exists('settings_global')) {
      // Set the setting names to retrieve with their default values.
      $settings_to_store = array(
        'timeadjust' => '+0 hours',
        'DBVersion' => 0,
      );
      $previous_db = limesurvey_sync_db_open();
      // Retrieve the limeSurvey global settings values.
      $gs_query = db_query("SELECT stg_name, stg_value FROM " . limesurvey_sync_table('settings_global'));
      while ($setting = db_fetch_array($gs_query)) {
        if (array_key_exists($setting['stg_name'], $settings_to_store)) {
          $ls_gs[$setting['stg_name']] = $setting['stg_value'];
        }
      }
      limesurvey_sync_db_close($previous_db);
      // Make sure the keys exist (avoid to return an error if the table does
      // not exist).
      foreach ($settings_to_store as $name => $default_value) {
        if (!isset($ls_gs[$name])) {
          $ls_gs[$name] = $default_value;
        }
      }
    }
    else {
      return FALSE;
    }
  }
  return $ls_gs;
}

/**
 * Fix for the LimeSurvey storing date system.
 *
 * The LimeSurvey software stores datetime values including the timezone value
 * whereas the Drupal system stores timestamp value and apply then the timezone
 * parameter. This function returns the correct datetime value applying the
 * LimeSurvey timezone from a timestamp. As long as the Limesurvey site and the
 * Drupal site are using the same timezone (this is checked by the
 * synchronization setting page), there is no adaptations.
 *
 * @param $timestamp
 *   A integer. The timestamp to be convert to the LimeSurvey datetime system,
 *   default is NULL corresponding to the current time.
 *
 * @return
 *   A string, the english formated corresponding date.
 */
function limesurvey_sync_fixdate_drupal_to_ls($timestamp = NULL) {
  if ($timestamp == NULL) {
    $timestamp = time();
  }
  $delta_time = variable_get('ls_settings', array('ls_delta_time' => 0));
  $drupal_timezone = variable_get('date_default_timezone', 0);
  $server_timezone = limesurvey_sync_get_timezone_offset(@date_default_timezone_get());
  // The date() php function uses date_default_timezone_get() in order to
  // determine the timezone.
  // Issue : if php < 5.2 or if the server timezone is not UTC, the date can
  // not be calculated ! As a workaround, the admin user may edit the time
  // difference on the LimeSurvey synchronization setting page in order to fix
  // this limitation on drupal 6.
  // Overwrite the value if it fails.
  $server_timezone = ($server_timezone !== FALSE) ? $server_timezone : 0;
  $delta_tz = $drupal_timezone - $server_timezone;
  return date('Y-m-d H:i:s', ($timestamp + $delta_tz + $delta_time['ls_delta_time']));
}

/**
 * Convert english date to the synchronized timestamp.
 *
 * As an example, convert '2012-01-01 10:22:00' to 1325409720, adjusting it if
 * Drupal and LimeSurvey servers are not synchronized or use a specific
 * timezone.
 *
 * @param $english_date
 *   A string. The english formated date.
 *
 * @return
 *   A numeric. The Drupal timestamp corresponding value.
 */
function limesurvey_sync_fixdate_ls_to_drupal($english_date) {
  if (empty($english_date)) {
    return '';
  }
  $delta_time = variable_get('ls_settings', array('ls_delta_time' => 0));
  $drupal_timezone = variable_get('date_default_timezone', 0);
  return (strtotime($english_date) - $drupal_timezone - $delta_time['ls_delta_time']);
}

/**
 * Append the offset to a LimeSurvey english date.
 *
 * This function is used to display non dynamic date that won't be adapted to
 * the user timezone. For instance : the submidate, or the startdate stored
 * into an html table. '2012-01-01 10:22:00' may returns
 * '2012-01-01 10:22:00 +0100'.
 *
 * @param $english_date
 *   A string. The LimeSurvey english formated date.
 *
 * @return
 *   A translated date string in the requested format.
 */
function limesurvey_sync_format_static_lsdate($english_date) {
  if (empty($english_date)) {
    return t('No date');
  }
  $timestamp = limesurvey_sync_fixdate_ls_to_drupal($english_date);
  $drupal_timezone_second = variable_get('date_default_timezone', 0);
  return format_date($timestamp, 'small', '', $drupal_timezone_second) . ' ' . limesurvey_sync_get_timezone_offset_string($drupal_timezone_second);
}

/**
 * Provide the LimeSurvey major version.
 *
 * The LimeSurvey version is retrieved by the DBVersion value stored into the
 * database. I don't use the value from the version.php file because it may be
 * not reachable if the LimeSurvey software is stored on an external server.
 * Note :
 * - the 1.92+ version build 120919 owns the 155.6 DBVersion,
 * - the v2.0+ build 121019 owns the 163 DBVersion.
 * I arbitrary assume that the LimeSurvey v2.0 have started from the 160
 * DBVersion.
 *
 * @return
 *   A string, '1.x' or '2.x', or FALSE if it fails.
 */
function limesurvey_sync_lsversion() {
  $ls_v2_dbversion = 160;
  static $lsversion = NULL;
  // Use cache.
  if ($lsversion ===  NULL) {
    $ls_gs = limesurvey_sync_ls_global_settings();
    if (!empty($ls_gs['DBVersion'])) {
      $lsversion = ($ls_gs['DBVersion'] < $ls_v2_dbversion) ? '1.x' : '2.x';
    }
    else {
      // If there are some synchronization troubles, act as if it is the
      // LimeSurvey v2.x version (the actual stable version).
      $lsversion = '2.x';
    }
  }
  return $lsversion;
}

/**
 * Provide url to specific pages of the LimeSurvey admin interface.
 *
 * This function is safe, sanitize values that may be displayed with no more
 * control (via form_set_error() as an example).
 *
 * @param $sid
 *   An numeric. Corresponding to the survey reference ID. Default is NULL.
 * @param $page
 *   A string. The page. Check the function for available values. Default is
 *   'admin'.
 * @param $anchor
 *   An string. The anchor value. Default is empty.
 * @param $absolute
 *   A boolean. For absolute url. Default is TRUE.
 *
 * @return
 *   A string, The corresponding url.
 */
function limesurvey_sync_admin_url($sid = NULL, $page = 'admin', $anchor = '', $absolute = TRUE) {
  $options = array(
    'absolute' => $absolute,
    'external' => TRUE,
  );

  $ls_v2 = (limesurvey_sync_lsversion() != '1.x');
  $admin_page = ($ls_v2) ? '/index.php/admin/' : '/admin/admin.php';
  $ls_settings = variable_get('ls_settings', array('ls_path' => ''));
  $url = check_plain($ls_settings['ls_path'] . $admin_page);
  if ($absolute) {
    module_load_include('inc', 'limesurvey_sync', 'limesurvey_sync');
    $url = limesurvey_sync_rel2abs($url);
  }

  switch ($page) {
    case 'globalsettings':
      if ($ls_v2) {
        $url .= 'globalsettings';
      }
      else {
        $options['query']['action'] = 'globalsettings';
      }
      break;
    case 'newsurvey':
      if ($ls_v2) {
        $url .= 'survey/newsurvey';
      }
      else {
        $options['query']['action'] = 'newsurvey';
      }
      break;
    case 'listsurveys':
      if ($ls_v2) {
        $url .= 'survey/index';
      }
      else {
        $options['query']['action'] = 'listsurveys';
      }
      break;
      // Below : need survey ID.
    case 'admin':
      if ($ls_v2) {
        $url .= 'survey/view';
      }
      break;
    case 'editsurveysettings':
      if ($ls_v2) {
        $url .= 'survey/editsurveysettings';
      }
      else {
        $options['query']['action'] = 'editsurveysettings';
      }
      break;
    case 'activate':
      if ($ls_v2) {
        $url .= 'survey/activate';
      }
      else {
        $options['query']['action'] = 'activate';
      }
      break;
    case 'statistics':
      if ($ls_v2) {
        $url .= 'statistics/index';
      }
      else {
        $options['query']['action'] = 'statistics';
      }
      break;
    case 'tokens':
      if ($ls_v2) {
        $url .= 'tokens/index';
      }
      else {
        $options['query']['action'] = 'tokens';
      }
      break;
    case 'browse':
      if ($ls_v2) {
        $url .= 'responses/index';
      }
      else {
        $options['query']['action'] = 'browse';
      }
      break;
    case 'editsurveylocalesettings':
      if ($ls_v2) {
        $url .= 'survey/editlocalsettings';
      }
      else {
        $options['query']['action'] = 'editsurveylocalesettings';
      }
      break;
  }

  switch ($anchor) {
    case 'presentation':
      $options['fragment'] = 'presentation';
      break;
    case 'tokens':
      $options['fragment'] = 'tokens';
      break;
    case 'general':
      $options['fragment'] = 'general';
      break;
    case 'publication':
      $options['fragment'] = 'publication';
      break;
    case 'notification':
      $options['fragment'] = 'notification';
      break;
    case 'copy':
      $options['fragment'] = 'copy';
      break;
  }
  if (preg_match('#^edittxtele[0-9]{1,2}$#', $anchor) || preg_match('#^url_[a-zA-Z\-]{0,12}$#', $anchor)) {
    // Handle : 'edittxtele0' stands for default language, 'edittxtele1' for
    // the first additional language, etc... And the 'url_en', url_fr' for
    // targeting the end url textfield; thanks to the
    // limesurvey_sync_localsettings_anchor() function.
    $options['fragment'] = $anchor;
  }

  switch ($page) {
    case 'admin':
    case 'editsurveysettings':
    case 'activate':
    case 'statistics':
    case 'tokens':
    case 'browse':
    case 'editsurveylocalesettings':
      if (!empty($sid)) {
        if ($ls_v2) {
          $url .= '/surveyid/' . $sid;
        }
        else {
          $options['query']['sid'] =  $sid;
        }
      }
      else {
        // If sid id not provide, those pages do not exist.
        // Go to the defaut admin page instead.
        if ($ls_v2) {
          $url = $ls_settings['ls_path'] . $admin_page . 'survey/index';
        }
        else {
          unset($options['query']['action'], $options['fragment']);
        }
      }
      break;
    default:
  }

  // Set 'sid' before 'action' in the url.
  if (isset($options['query']) && is_array($options['query'])) {
    krsort($options['query']);
  }

  return url($url, $options);
}

/**
 * Check if a table exists on the LimeSurvey database.
 *
 * This is an adaptation of the db_table_exists() core function. Curly braces
 * are disabled because we want to use the LimeSurvey database prefix instead
 * of the Drupal database prefix.
 *
 * @param $table
 *   A string. Corresponding to the LimeSurvey table generic name.
 * @param $sid
 *   (optional) An integer. Corresponding to the survey ID reference. Default
 *   is empty.
 * @param $ls_settings
 *   (optional) An array. Corresponding to the LimeSurvey database connecting
 *   informations. Default is empty, current settings will be used.
 *
 * @return
 *   TRUE if the table exists, and FALSE if the table does not exist.
 */
function limesurvey_sync_db_table_exists($table, $sid = '', $ls_settings = array()) {
  global $db_type;
  // Activate the LimeSurvey database.
  $previous_db = limesurvey_sync_db_open();
  switch ($db_type) {
    case 'mysqli':
    case 'mysql':
      // HERE : no curly braces.
      $table_exists = (bool) db_fetch_object(db_query("SHOW TABLES LIKE '" . db_escape_table(limesurvey_sync_table($table, $sid, $ls_settings)) . "'"));
      break;
    case 'pgsql':
      // HERE : no curly braces.
      $table_exists = (bool) db_result(db_query("SELECT COUNT(*) FROM pg_class WHERE relname = '" . db_escape_table(limesurvey_sync_table($table, $sid, $ls_settings)) . "'"));
      break;
    default:
      $table_exists = FALSE;
  }
  // Switch back to the Drupal database.
  limesurvey_sync_db_close($previous_db);
  return $table_exists;
}

